<?php

/**
 * ochStrictrouter - System plugin
 *
 * @version     0.2.1
 * @package     Joomla
 * @subpackage  plg_system_ochstrictrouter
 *
 * @author      Ruud van Lent <info@onlinecommunityhub.nl>
 * @copyright   Copyright (C) 2019 - 2021 Onlinecommunityhub (X-ceed B.V.). All rights reserved.
 * @license     GNU/GPL version 3 or later
 * @link        https://onlinecommunityhub.nl
 */


// No direct access
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\Utilities\IpHelper;
use OCH\ochStrictrouter\OchHelper;

// Include the helper functions only once
JLoader::register('OCH\ochStrictrouter\OchHelper', __DIR__ . '/helpers/ochhelper.php');
JLoader::register('OchStrictrouterHelper', __DIR__ . '/helpers/helper.php');
require_once __DIR__ . '/libraries/vendor/autoload.php';

/**
 * Plug-in to add ads to a website
 *
 * @since  0.0.0
 */
class PlgSystemochStrictrouter extends CMSPlugin
{
	/**
	 * Load language file for front-end translations
	 *
	 * @var boolean
	 */
	protected $autoloadLanguage = true;

	/**
	 * A Registry object holding the parameters for the plugin
	 *
	 * @var    Registry
	 * @since  1.5
	 */
	public $params;

	/**
	 * Application object.
	 *
	 * @var    Joomla\CMS\Application\CMSApplication
	 * @since  3.3
	 */
	protected $app;

	/**
	 * Factory Document
	 *
	 * @var object
	 */
	protected $document;

	/**
	 * Database object.
	 *
	 * @var    JDatabaseDriver
	 * @since  3.8.0
	 */
	protected $db;

	/**
	 * Plugin version
	 *
	 * @var string
	 */
	protected $version = '0.2.1';

	/**
	 * Plugin debug
	 *
	 * @var string
	 */
	protected $debug = '0';

	/**
	 * Constructor
	 *
	 * @param   object  $subject  The object to observe
	 * @param   array   $config   An optional associative array of configuration settings.
	 *                            Recognized key values include 'name', 'group', 'params', 'language'
	 *                            (this list is not meant to be comprehensive).
	 *
	 * @since   1.5
	 */
	public function __construct(&$subject, $config = array())
	{
		$this->document = Factory::getDocument();

		parent::__construct($subject, $config);

		// Enable and set debugging level
		if ($this->params->get('ochstrictrouterDebug', '0') || JDEBUG)
		{
			$this->debug = '1';
			Log::addLogger(array('text_file' => 'plg_system_ochstrictrouter-log.php'), Log::ALL, array('ochstrictrouter'));
		}
		else
		{
			$this->debug = '0';
			Log::addLogger(array('text_file' => 'plg_system_ochstrictrouter-log.php'), Log::ALL & ~Log::INFO, array('ochstrictrouter'));
		}
	}

	/**
	 * Redirect to 404 page when article ID and Article Alias do not match
	 *
	 * @param   string   $context  The context of the content being passed to the plugin
	 * @param   object   $row      The article object
	 * @param   object   $params   The article params
	 * @param   integer  $page     The 'page' number
	 *
	 * @return  boolean
	 *
	 * @since   1.6
	 */
	public function onContentBeforeDisplay($context, &$row, &$params, $page = 0)
	{
		if (!OchStrictrouterHelper::checkExecute($this->app, $this->params))
		{
			return false;
		}

		if ($context == 'com_content.article' && $this->params->get('catchRedirect', '0'))
		{
			$query  = [];
			$string = 'index.php?';
			$input  = $this->app->input;

			$query['option'] = $input->get('option', '');
			$query['view']   = $input->get('view', '');
			$query['catid']  = $input->get('catid', '');
			$query['id']     = $input->get('id', '');
			$query['lang']   = $input->get('lang', '');
			$query['Itemid'] = $input->get('Itemid', '');

			if ($query['catid'] == 0)
			{
				$dbquery = $this->db->getQuery(true);
				$dbquery->select('catid')
					->from($this->db->quoteName('#__content'))
					->where($this->db->quoteName('id') . ' = ' . $this->db->quote($query['id']));

				$this->db->setQuery($dbquery);
				$query['catid'] = $this->db->loadResult();
			}

			foreach ($query as $key => $value)
			{
				if (!empty($value))
				{
					$string .= $key . '=' . $value . '&';
				}
			}

			$realUrl = Route::_(trim($string, '&'));
			$url     = Uri::getInstance()->toString();

			if (stripos($url, $realUrl) === false)
			{
				// We have a mismatch, let's take action!
				$referer   = isset($_SERVER['HTTP_REFERER']) ? strip_tags($_SERVER['HTTP_REFERER']) : '-';
				$ipAddress = IpHelper::getIp();
				$hostName  = gethostbyaddr($ipAddress);

				$redirectAction = $this->params->get('redirectAction', '404');

				Log::add(Text::sprintf('PLG_SYSTEM_OCHSTRICTROUTER_LOG_MESSAGE_CATCHEDURLREFERER', $url, $referer, $hostName), Log::WARNING, 'ochstrictrouter');

				switch ($redirectAction)
				{
					case 'report':
						return;
						break;

					case '404':
						throw new Exception(JText::_('JERROR_PAGE_NOT_FOUND'), 404);
						break;

					case '410':
						throw new Exception(JText::_('JERROR_PAGE_NOT_FOUND'), 410);
						break;

					case 'redirect':
						$redirect = Route::link('site', 'index.php?Itemid=' . $this->params->get('redirectLocation'), true, Route::TLS_IGNORE, true);
						$this->app->redirect($redirect, 301);
						break;
				}

				// If everything fails
				die('Borked...');
			}
		}
	}

	/**
	 * Handle adding download id to package download request.
	 *
	 * @param   string  $url      url from which package is going to be downloaded
	 * @param   array   $headers  headers to be sent along the download request (key => value format)
	 *
	 * @return boolean true if download id added to request false otherwise
	 *
	 * @since   3.0
	 */
	public function onInstallerBeforePackageDownload(&$url, &$headers)
	{
		$package                 = new stdClass;
		$package->url            = $url;
		$package->headers        = $headers;
		$package->languagePrefix = 'PLG_SYSTEM_OCHSTRICTROUTER';
		$package->downloadId     = $this->params->get('ochstrictrouter_downloadid', '');
		$package->plugin         = PluginHelper::getPlugin('system', 'ochstrictrouter');

		if (OchHelper::isJoomla4() && empty($package->downloadId))
		{
			$uri = clone Uri::getInstance($package->url);

			$package->downloadId = $uri->getVar('key', '');
		}

		$package = OchHelper::prepareUpdate($package);

		if ($package === false)
		{
			// Something went wrong getting the download id, bail now
			return false;
		}

		// Create the url string
		$url     = $package->url;
		$headers = $package->headers;

		return true;
	}

	/**
	 * Function to validate the subscription download id
	 *
	 * @param   string   $context  The content the function is called from
	 * @param   object   $table    The table holding the configured parameters
	 * @param   boolean  $isNew    boolean to determine if extension is new
	 *
	 * @return  void
	 */
	public function onExtensionAfterSave($context, $table, $isNew)
	{
		$element = 'ochstrictrouter';

		// $this->params holds the saved values, not the changed values so need to read and json_decode the updated values
		$paramsUpdated = json_decode($table->params, true);
		$downloadId    = isset($paramsUpdated['ochstrictrouter_downloadid']) ? $paramsUpdated['ochstrictrouter_downloadid'] : '';

		// Run only on plugins configuration edit form (back-end) and when download id is set
		if (
			$this->app->isClient('site')
			|| ($context != 'com_plugins.plugin' && $context != 'com_ochseotoolbox.plugin' && $context != 'com_ochblog.plugin' && $context != 'com_ochsubscriptions.plugin')
			|| $table->element != $element
			|| !isset($table->params)
		)
		{
			return;
		}

		$result = OchHelper::validateDownloadId(
			'https://onlinecommunityhub.nl/index.php?option=com_ochsubscriptions&task=updater.validateKey&element=' . $element,
			$element,
			$downloadId,
			true,
			false,
			true,
			'PLG_SYSTEM_OCHSTRICTROUTER'
		);
	}

	/**
	 * Function to interact with OnlineCommunityHub component's control panels
	 *
	 * @return  array  $cpanel
	 */
	public function onShowOchCpanel()
	{
		$element          = 'ochstrictrouter';
		$downloadId       = $this->params->get('ochstrictrouter_downloadid', '');
		$validationResult = OchHelper::validateDownloadId(
			'https://onlinecommunityhub.nl/index.php?option=com_ochsubscriptions&task=updater.validateKey&element=' . $element,
			$element,
			$downloadId,
			true,
			true,
			false
		);

		// Get this plugin's data object
		$plugin = PluginHelper::getPlugin('system', $element);

		$cpanel['version']      = $this->version;
		$cpanel['name']         = Text::_('PLG_SYSTEM_OCHSTRICTROUTER');
		$cpanel['description']  = Text::_('PLG_SYSTEM_OCHSTRICTROUTER_TOOLBOXTOOLTIP');
		$cpanel['image']        = Uri::root(true) . '/media/plg_system_ochstrictrouter/images/ochstrictrouter125.jpg';
		$cpanel['homepage']     = 'https://onlinecommunityhub.nl/joomla-extensions/ochstrictrouter';
		$cpanel['jed']          = 'https://extensions.joomla.org/extensions/extension/ads-a-affiliates/ochstrictrouter/';
		$cpanel['id']           = (int) $plugin->id;
		$cpanel['downloadid']   = $downloadId;
		$cpanel['subscription'] = $validationResult;
		$cpanel['language']     = '';

		return $cpanel;
	}
}
